@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Roberto Longobardi
+# Copyright (C) 2010-2015 Roberto Longobardi
 # 
 # This file is part of the Test Manager plugin for Trac.
 # 
@@ -20,37 +20,33 @@
 # If not, see <http://www.gnu.org/licenses/>.
 #
 
-import csv
-import json
-import os
-import pkg_resources
-import re
-import shutil
-import sys
-import time
-import traceback
-
-from datetime import datetime
-from operator import itemgetter
 from StringIO import StringIO
+import csv
+from datetime import datetime
+import json
+from operator import itemgetter
+import os
+import re
 
-from trac.core import *
+from trac.core import Component, implements, TracError
+from trac.db.api import with_transaction
 from trac.mimeview.api import Context
-from trac.perm import IPermissionRequestor, PermissionError
-from trac.resource import Resource, IResourceManager, render_resource_link, get_resource_url
-from trac.util import get_reporter_id, format_datetime, format_date
+from trac.perm import IPermissionRequestor
+from trac.resource import IResourceManager
+from trac.util import get_reporter_id, format_datetime
 from trac.util.datefmt import utc
 from trac.web.api import IRequestHandler
-from trac.web.chrome import add_notice, add_warning, add_stylesheet
-from trac.wiki.formatter import Formatter, format_to_html
+from trac.web.chrome import add_notice, add_warning
+from trac.wiki.formatter import Formatter
 from trac.wiki.model import WikiPage
 from trac.wiki.parser import WikiParser
 
-from tracgenericclass.model import GenericClassModelProvider
-from tracgenericclass.util import *
-
 from testmanager.model import TestCatalog, TestCase, TestCaseInPlan, TestPlan
-from testmanager.util import *
+from testmanager.util import get_page_title
+from tracgenericclass.model import GenericClassModelProvider
+from tracgenericclass.util import formatExceptionInfo, from_any_timestamp, \
+    upload_file_to_subdir, get_dictionary_from_string
+
 
 try:
     from trac.util.translation import domain_functions
@@ -772,7 +768,7 @@ class TestManagerSystem(Component):
 
                     try:
                         # Copy the test plan properties into a new test plan
-                        new_tp = TestPlan(self.env, id, tp['catid'], tp['page_name'], new_name, author, True, False)
+                        new_tp = TestPlan(self.env, id, tp['catid'], tp['page_name'], new_name, author, 1, 0)
                         new_tp.insert()
                         
                         # If needed, clone the test cases in the plan, with a default status 
@@ -1113,13 +1109,43 @@ class TestManagerSystem(Component):
             self.env.log.error(formatExceptionInfo())
             return None
 
+    def get_default_tc_template_id(self):
+        """ get default TestCase template id """
+        try:
+            return self.get_config_property('TEST_CASE_DEFAULT_TEMPLATE')
+
+        except:
+            self.env.log.error("Error getting default test case template id")
+            self.env.log.error(formatExceptionInfo())
+            return None
+
+    def get_default_tc_template(self):
+        """ get default TestCase template """
+        try:
+            # first get template id from testconfig
+            t_id = self.get_config_property('TEST_CASE_DEFAULT_TEMPLATE')
+            if not t_id:
+                return ''
+
+            # now get template
+            result = self.get_template_by_id(t_id)
+            if not result:
+                return ''
+                
+            return result['content']
+
+        except:
+            self.env.log.error("Error getting default test case template")
+            self.env.log.error(formatExceptionInfo())
+            return None
+
     def get_tc_template_id_for_catalog(self, t_cat_id):
         """ get test case template for catalog with specified id """
         try:
             return self.get_config_property('TC_TEMPLATE_FOR_TCAT_' + t_cat_id)
 
         except:
-            self.env.log.error("Error getting default test catalog template id")
+            self.env.log.error("Error getting default test case template id")
             self.env.log.error(formatExceptionInfo())
             return None
 
@@ -1132,8 +1158,12 @@ class TestManagerSystem(Component):
 
             # now get Template ID
             t_id = self.get_tc_template_id_for_catalog(t_cat_id)
-            if not t_id:
-                return ''
+            
+            if not t_id or t_id == '' or t_id == '0':
+                t_id = self.get_default_tc_template_id()
+
+                if not t_id or t_id == '':
+                    return ''
 
             # and finally get the template
             result = self.get_template_by_id(t_id)
@@ -1287,15 +1317,13 @@ class TestManagerSystem(Component):
         
         db = self.env.get_read_db()
         cursor = db.cursor()
-        cursor.execute("SELECT * from testcatalog")
+        cursor.execute("SELECT id, page_name from testcatalog")
         items = []
-        for row in cursor:
-            c_id = row[0]
-            c_name = row[1]
+        for c_id, c_name in cursor:
             wikipage = WikiPage(self.env, c_name)
             c_title = get_page_title(wikipage.text)
-            c_template = self.get_tc_template_id_for_catalog(c_id)
-            cat = {'id': c_id, 'name': c_name, 'title': c_title, 'template': c_template}
+            c_template_id = self.get_tc_template_id_for_catalog(c_id)
+            cat = {'id': c_id, 'name': c_name, 'title': c_title, 'template': c_template_id}
             items.append(cat)
             
         return sorted(items, key=itemgetter('title'))
