@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Roberto Longobardi
+# Copyright (C) 2010-2015 Roberto Longobardi
 # 
 # This file is part of the Test Manager plugin for Trac.
 # 
@@ -20,21 +20,16 @@
 # If not, see <http://www.gnu.org/licenses/>.
 #
 
-import re
-import time
-
-from datetime import datetime
-from trac.core import *
-from trac.resource import Resource, render_resource_link, get_resource_url
+from genshi.builder import tag
+from trac.core import Interface, Component, implements, ExtensionPoint, \
+    TracError
+from trac.resource import Resource, get_resource_url
 from trac.util import get_reporter_id
-from trac.util.translation import _, N_, gettext
+from trac.util.translation import _
 from trac.web.api import IRequestHandler
 from trac.web.chrome import ITemplateProvider
 
-from genshi.builder import tag
-
-from tracgenericclass.util import *
-
+from tracgenericclass.util import formatExceptionInfo
 from tracgenericworkflow.model import ResourceWorkflowState
 
 
@@ -44,7 +39,7 @@ class IWorkflowTransitionListener(Interface):
     when objects transition between states.
     """
 
-    def object_transition(res_wf_state, resource, action, old_state, new_state):
+    def object_transition(self, res_wf_state, resource, action, old_state, new_state):
         """
         Called when an object has transitioned to a new state.
 
@@ -62,7 +57,7 @@ class IWorkflowTransitionAuthorization(Interface):
     based on the object and the current and new states.
     """
 
-    def is_authorized(res_wf_state, resource, action, old_state, new_state):
+    def is_authorized(self, res_wf_state, resource, action, old_state, new_state):
         """
         Called before allowing the transition.
         Return True to allow for the transition, False to deny it.
@@ -80,14 +75,14 @@ class IWorkflowOperationProvider(Interface):
     custom workflow operations.
     """
 
-    def get_implemented_operations():
+    def get_implemented_operations(self):
         """
         Return custom actions provided by the component.
 
         :rtype: `basestring` generator
         """
 
-    def get_operation_control(req, action, operation, res_wf_state, resource):
+    def get_operation_control(self, req, action, operation, res_wf_state, resource):
         """
         Asks the provider to provide UI control to let the User 
         perform the specified operation on the given resource.
@@ -105,7 +100,7 @@ class IWorkflowOperationProvider(Interface):
                  with the operation hint.
         """
         
-    def perform_operation(req, action, operation, old_state, new_state, res_wf_state, resource):
+    def perform_operation(self, req, action, operation, old_state, new_state, res_wf_state, resource):
         """
         Perform the specified operation on the given resource, which 
         has transitioned from the given old to the given new state.
@@ -420,7 +415,7 @@ class ResourceWorkflowSystem(Component):
                             try:
                                 rws.save_changes(author, "State changed")
                             except:
-                                self.log.debug("Error saving the resource %s with id %s" % (realm, id))
+                                self.log.debug("Error saving the resource %s with id %s" % (res_realm, id))
                         else:
                             TracError("Resource with id %s has already changed state in the meanwhile. Current state is %s." % (id, rws['state']))
                 else:
